""" B{BigJob-based Implementation of Pilot-API} 

This package contains the implementations of the Pilot API objects: L{PilotComputeService}, L{PilotCompute}, L{PilotDataService}, L{PilotData}, L{ComputeDataService}.

"""